/*
 * XmlApi.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "XmlApi.hpp"

namespace alib {

const label::LabelFromXMLParser FromXMLParsers::labelParser;
const alphabet::SymbolFromXMLParser FromXMLParsers::symbolParser;
const regexp::RegExpFromXMLParser FromXMLParsers::regexpParser;
const string::StringFromXMLParser FromXMLParsers::stringParser;
const automaton::AutomatonFromXMLParser FromXMLParsers::automatonParser;
const grammar::GrammarFromXMLParser FromXMLParsers::grammarParser;
const exception::ExceptionFromXMLParser FromXMLParsers::exceptionParser;
const alib::ObjectFromXMLParser FromXMLParsers::objectParser;
const container::ContainerFromXMLParser FromXMLParsers::containerParser;
const primitive::PrimitiveFromXMLParser FromXMLParsers::primitiveParser;

const label::LabelToXMLComposer ToXMLComposers::labelComposer;
const alphabet::SymbolToXMLComposer ToXMLComposers::symbolComposer;
const regexp::RegExpToXMLComposer ToXMLComposers::regexpComposer;
const string::StringToXMLComposer ToXMLComposers::stringComposer;
const automaton::AutomatonToXMLComposer ToXMLComposers::automatonComposer;
const grammar::GrammarToXMLComposer ToXMLComposers::grammarComposer;
const alib::ObjectToXMLComposer ToXMLComposers::objectComposer;
const exception::ExceptionToXMLComposer ToXMLComposers::exceptionComposer;
const container::ContainerToXMLComposer ToXMLComposers::containerComposer;
const primitive::PrimitiveToXMLComposer ToXMLComposers::primitiveComposer;

const ToXMLComposers ToXMLComposers::toXMLComposers;

const std::string Names::ALIB_VOID = "Void";
const std::string Names::CONTAINER_OBJECTS_SET = "ObjectsSet";
const std::string Names::CONTAINER_OBJECTS_VECTOR = "ObjectsVector";
const std::string Names::CONTAINER_OBJECTS_PAIR = "ObjectsPair";
const std::string Names::CONTAINER_OBJECTS_MAP = "ObjectsMap";
const std::string Names::EXCEPTION_ALIB_EXCEPTION = "AlibException";
const std::string Names::ALPHABET_LABELED_SYMBOL = "LabeledSymbol";
const std::string Names::ALPHABET_BLANK_SYMBOL = "BlankSymbol";
const std::string Names::ALPHABET_BOTTOM_OF_THE_STACK_SYMBOL = "BottomOfTheStackSymbol";
const std::string Names::ALPHABET_END_SYMBOL = "EndSymbol";
const std::string Names::ALPHABET_RANKED_SYMBOL = "RankedSymbol";
const std::string Names::ALPHABET_BAR_SYMBOL = "BarSymbol";
const std::string Names::ALPHABET_RANKED_BAR_SYMBOL = "RankedBarSymbol";
const std::string Names::ALPHABET_SUBTREE_WILDCARD_SYMBOL = "SubtreeWildcardSymbol";
const std::string Names::ALPHABET_SYMBOL_SET_SYMBOL = "SymbolSetSymbol";
const std::string Names::ALPHABET_SYMBOL_PAIR_SYMBOL = "SymbolPairSymbol";
const std::string Names::ALPHABET_UNIQUE_SYMBOL = "UniqueSymbol";
const std::string Names::ALPHABET_START_SYMBOL = "StartSymbol";
const std::string Names::AUTOMATON_EPSILON_NFA = "EpsilonNFA";
const std::string Names::AUTOMATON_MULTI_INITIAL_STATE_NFA = "MultiInitialStateNFA";
const std::string Names::AUTOMATON_NFA = "NFA";
const std::string Names::AUTOMATON_DFA = "DFA";
const std::string Names::AUTOMATON_EXTENDED_NFA = "ExtendedNFA";
const std::string Names::AUTOMATON_COMPACT_NFA = "CompactNFA";
const std::string Names::AUTOMATON_DPDA = "DPDA";
const std::string Names::AUTOMATON_SINGLE_POP_DPDA = "SinglePopDPDA";
const std::string Names::AUTOMATON_INPUT_DRIVEN_NPDA = "InputDrivenNPDA";
const std::string Names::AUTOMATON_VISIBLY_PUSHDOWN_DPDA = "VisiblyPushdownDPDA";
const std::string Names::AUTOMATON_VISIBLY_PUSHDOWN_NPDA = "VisiblyPushdownNPDA";
const std::string Names::AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_DPDA = "RealTimeHeightDeterministicDPDA";
const std::string Names::AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_NPDA = "RealTimeHeightDeterministicNPDA";
const std::string Names::AUTOMATON_NPDA = "NPDA";
const std::string Names::AUTOMATON_SINGLE_POP_NPDA = "SinglePopNPDA";
const std::string Names::AUTOMATON_ONE_TAPE_DTM = "OneTapeDTM";
const std::string Names::AUTOMATON_STATE = "State";
const std::string Names::GRAMMAR_LEFT_LG = "LeftLG";
const std::string Names::GRAMMAR_LEFT_RG = "LeftRG";
const std::string Names::GRAMMAR_RIGHT_LG = "RightLG";
const std::string Names::GRAMMAR_RIGHT_RG = "RightRG";
const std::string Names::GRAMMAR_LG = "LG";
const std::string Names::GRAMMAR_CFG = "CFG";
const std::string Names::GRAMMAR_EPSILON_FREE_CFG = "EpsilonFreeCFG";
const std::string Names::GRAMMAR_CNF = "CNF";
const std::string Names::GRAMMAR_GNF = "GNF";
const std::string Names::GRAMMAR_CSG = "CSG";
const std::string Names::GRAMMAR_NON_CONTRACTING_GRAMMAR = "NonContractingGrammar";
const std::string Names::GRAMMAR_CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR = "ContextPpreservingUnrestrictedGrammar";
const std::string Names::GRAMMAR_UNRESTRICTED_GRAMMAR = "UnrestrictedGrammar";
const std::string Names::LABEL_PRIMITIVE_LABEL = "PrimitiveLabel";
const std::string Names::LABEL_HEXAVIGESIMAL_LABEL = "HexavigesimalLabel";
const std::string Names::LABEL_OBJECT_LABEL = "ObjectLabel";
const std::string Names::LABEL_LABEL_SET_LABEL = "LabelSetLabel";
const std::string Names::LABEL_LABEL_PAIR_LABEL = "LabelPairLabel";
const std::string Names::LABEL_UNIQUE_LABEL = "UniqueLabel";
const std::string Names::REGEXP_UNBOUNDED_REGEXP = "UnboundedRegExp";
const std::string Names::REGEXP_FORMAL_REGEXP = "FormalRegExp";
const std::string Names::STRING_LINEAR_STRING = "LinearString";
const std::string Names::STRING_CYCLIC_STRING = "CyclicString";
const std::string Names::STRING_EPSILON = "Epsilon";
const std::string Names::PRIMITIVE_PRIMITIVE = "Primitive";
const std::string Names::PRIMITIVE_STRING = "String";
const std::string Names::PRIMITIVE_INTEGER = "Integer";
const std::string Names::PRIMITIVE_CHARACTER = "Character";
const std::string Names::PRIMITIVE_UNSIGNED = "Unsigned";
const std::string Names::PRIMITIVE_BOOL = "Bool";

Void xmlApi<Void>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::objectParser.parseVoid(input);
}

bool xmlApi<Void>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALIB_VOID);
}

void xmlApi<Void>::compose(std::list<sax::Token>& output, const Void& data) {
	ToXMLComposers::objectComposer.compose(output, data);
}

container::Container xmlApi<container::Container>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseContainer(input);
}

bool xmlApi<container::Container>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.first(input);
}

void xmlApi<container::Container>::compose(std::list<sax::Token>& output, const container::Container& data) {
	ToXMLComposers::containerComposer.compose(output, data);
}

container::ObjectsSet xmlApi<container::ObjectsSet>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseObjectsSet(input);
}

bool xmlApi<container::ObjectsSet>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::CONTAINER_OBJECTS_SET);
}

void xmlApi<container::ObjectsSet>::compose(std::list<sax::Token>& output, const container::ObjectsSet& data) {
	ToXMLComposers::containerComposer.compose(output, data);
}

container::ObjectsVector xmlApi<container::ObjectsVector>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseObjectsVector(input);
}

bool xmlApi<container::ObjectsVector>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::CONTAINER_OBJECTS_VECTOR);
}

void xmlApi<container::ObjectsVector>::compose(std::list<sax::Token>& output, const container::ObjectsVector& data) {
	ToXMLComposers::containerComposer.compose(output, data);
}

container::ObjectsPair xmlApi<container::ObjectsPair>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseObjectsPair(input);
}

bool xmlApi<container::ObjectsPair>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::CONTAINER_OBJECTS_PAIR);
}

void xmlApi<container::ObjectsPair>::compose(std::list<sax::Token>& output, const container::ObjectsPair& data) {
	ToXMLComposers::containerComposer.compose(output, data);
}

container::ObjectsMap xmlApi<container::ObjectsMap>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseObjectsMap(input);
}

bool xmlApi<container::ObjectsMap>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::CONTAINER_OBJECTS_MAP);
}

void xmlApi<container::ObjectsMap>::compose(std::list<sax::Token>& output, const container::ObjectsMap& data) {
	ToXMLComposers::containerComposer.compose(output, data);
}

alib::Object xmlApi<alib::Object>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::objectParser.parseObject(input);
}

bool xmlApi<alib::Object>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::objectParser.first(input);
}

void xmlApi<alib::Object>::compose(std::list<sax::Token>& output, const alib::Object& data) {
	ToXMLComposers::objectComposer.compose(output, data);
}

exception::AlibException xmlApi<exception::AlibException>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::exceptionParser.parseException(input);
}

bool xmlApi<exception::AlibException>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::EXCEPTION_ALIB_EXCEPTION);
}

void xmlApi<exception::AlibException>::compose(std::list<sax::Token>& output, const exception::AlibException& data) {
	ToXMLComposers::exceptionComposer.compose(output, data);
}

alphabet::Symbol xmlApi<alphabet::Symbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseSymbol(input);
}

bool xmlApi<alphabet::Symbol>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.first(input);
}

void xmlApi<alphabet::Symbol>::compose(std::list<sax::Token>& output, const alphabet::Symbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::LabeledSymbol xmlApi<alphabet::LabeledSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseLabeledSymbol(input);
}

bool xmlApi<alphabet::LabeledSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_LABELED_SYMBOL);
}

void xmlApi<alphabet::LabeledSymbol>::compose(std::list<sax::Token>& output, const alphabet::LabeledSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::BlankSymbol xmlApi<alphabet::BlankSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseBlankSymbol(input);
}

bool xmlApi<alphabet::BlankSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_BLANK_SYMBOL);
}

void xmlApi<alphabet::BlankSymbol>::compose(std::list<sax::Token>& output, const alphabet::BlankSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::BottomOfTheStackSymbol xmlApi<alphabet::BottomOfTheStackSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseBottomOfTheStackSymbol(input);
}

bool xmlApi<alphabet::BottomOfTheStackSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_BOTTOM_OF_THE_STACK_SYMBOL);
}

void xmlApi<alphabet::BottomOfTheStackSymbol>::compose(std::list<sax::Token>& output, const alphabet::BottomOfTheStackSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::EndSymbol xmlApi<alphabet::EndSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseEndSymbol(input);
}

bool xmlApi<alphabet::EndSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_END_SYMBOL);
}

void xmlApi<alphabet::EndSymbol>::compose(std::list<sax::Token>& output, const alphabet::EndSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::StartSymbol xmlApi<alphabet::StartSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseStartSymbol(input);
}

bool xmlApi<alphabet::StartSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_START_SYMBOL);
}

void xmlApi<alphabet::StartSymbol>::compose(std::list<sax::Token>& output, const alphabet::StartSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::RankedSymbol xmlApi<alphabet::RankedSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseRankedSymbol(input);
}

bool xmlApi<alphabet::RankedSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_RANKED_SYMBOL);
}

void xmlApi<alphabet::RankedSymbol>::compose(std::list<sax::Token>& output, const alphabet::RankedSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::BarSymbol xmlApi<alphabet::BarSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseBarSymbol(input);
}

bool xmlApi<alphabet::BarSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_BAR_SYMBOL);
}

void xmlApi<alphabet::BarSymbol>::compose(std::list<sax::Token>& output, const alphabet::BarSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::RankedBarSymbol xmlApi<alphabet::RankedBarSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseRankedBarSymbol(input);
}

bool xmlApi<alphabet::RankedBarSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_RANKED_BAR_SYMBOL);
}

void xmlApi<alphabet::RankedBarSymbol>::compose(std::list<sax::Token>& output, const alphabet::RankedBarSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::SubtreeWildcardSymbol xmlApi<alphabet::SubtreeWildcardSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseSubtreeWildcardSymbol(input);
}

bool xmlApi<alphabet::SubtreeWildcardSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_SUBTREE_WILDCARD_SYMBOL);
}

void xmlApi<alphabet::SubtreeWildcardSymbol>::compose(std::list<sax::Token>& output, const alphabet::SubtreeWildcardSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::SymbolSetSymbol xmlApi<alphabet::SymbolSetSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseSymbolSetSymbol(input);
}

bool xmlApi<alphabet::SymbolSetSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_SYMBOL_SET_SYMBOL);
}

void xmlApi<alphabet::SymbolSetSymbol>::compose(std::list<sax::Token>& output, const alphabet::SymbolSetSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::SymbolPairSymbol xmlApi<alphabet::SymbolPairSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseSymbolPairSymbol(input);
}

bool xmlApi<alphabet::SymbolPairSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_SYMBOL_PAIR_SYMBOL);
}

void xmlApi<alphabet::SymbolPairSymbol>::compose(std::list<sax::Token>& output, const alphabet::SymbolPairSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}

alphabet::UniqueSymbol xmlApi<alphabet::UniqueSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseUniqueSymbol(input);
}

bool xmlApi<alphabet::UniqueSymbol>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::ALPHABET_UNIQUE_SYMBOL);
}

void xmlApi<alphabet::UniqueSymbol>::compose(std::list<sax::Token>& output, const alphabet::UniqueSymbol& data) {
	ToXMLComposers::symbolComposer.compose(output, data);
}


automaton::Automaton xmlApi<automaton::Automaton>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseAutomaton(input);
}

bool xmlApi<automaton::Automaton>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.first(input);
}

void xmlApi<automaton::Automaton>::compose(std::list<sax::Token>& output, const automaton::Automaton& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::EpsilonNFA xmlApi<automaton::EpsilonNFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseEpsilonNFA(input);
}

bool xmlApi<automaton::EpsilonNFA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_EPSILON_NFA);
}

void xmlApi<automaton::EpsilonNFA>::compose(std::list<sax::Token>& output, const automaton::EpsilonNFA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::MultiInitialStateNFA xmlApi<automaton::MultiInitialStateNFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseMultiInitialStateNFA(input);
}

bool xmlApi<automaton::MultiInitialStateNFA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_MULTI_INITIAL_STATE_NFA);
}

void xmlApi<automaton::MultiInitialStateNFA>::compose(std::list<sax::Token>& output, const automaton::MultiInitialStateNFA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::NFA xmlApi<automaton::NFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseNFA(input);
}

bool xmlApi<automaton::NFA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_NFA);
}

void xmlApi<automaton::NFA>::compose(std::list<sax::Token>& output, const automaton::NFA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::DFA xmlApi<automaton::DFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseDFA(input);
}

bool xmlApi<automaton::DFA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_DFA);
}

void xmlApi<automaton::DFA>::compose(std::list<sax::Token>& output, const automaton::DFA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::ExtendedNFA xmlApi<automaton::ExtendedNFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseExtendedNFA(input);
}

bool xmlApi<automaton::ExtendedNFA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_EXTENDED_NFA);
}

void xmlApi<automaton::ExtendedNFA>::compose(std::list<sax::Token>& output, const automaton::ExtendedNFA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::CompactNFA xmlApi<automaton::CompactNFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseCompactNFA(input);
}

bool xmlApi<automaton::CompactNFA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_COMPACT_NFA);
}

void xmlApi<automaton::CompactNFA>::compose(std::list<sax::Token>& output, const automaton::CompactNFA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::DPDA xmlApi<automaton::DPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseDPDA(input);
}

bool xmlApi<automaton::DPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_DPDA);
}

void xmlApi<automaton::DPDA>::compose(std::list<sax::Token>& output, const automaton::DPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::SinglePopDPDA xmlApi<automaton::SinglePopDPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseSinglePopDPDA(input);
}

bool xmlApi<automaton::SinglePopDPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_SINGLE_POP_DPDA);
}

void xmlApi<automaton::SinglePopDPDA>::compose(std::list<sax::Token>& output, const automaton::SinglePopDPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::InputDrivenNPDA xmlApi<automaton::InputDrivenNPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseInputDrivenNPDA(input);
}

bool xmlApi<automaton::InputDrivenNPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_INPUT_DRIVEN_NPDA);
}

void xmlApi<automaton::InputDrivenNPDA>::compose(std::list<sax::Token>& output, const automaton::InputDrivenNPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::VisiblyPushdownDPDA xmlApi<automaton::VisiblyPushdownDPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseVisiblyPushdownDPDA(input);
}

bool xmlApi<automaton::VisiblyPushdownDPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_VISIBLY_PUSHDOWN_DPDA);
}

void xmlApi<automaton::VisiblyPushdownDPDA>::compose(std::list<sax::Token>& output, const automaton::VisiblyPushdownDPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::VisiblyPushdownNPDA xmlApi<automaton::VisiblyPushdownNPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseVisiblyPushdownNPDA(input);
}

bool xmlApi<automaton::VisiblyPushdownNPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_VISIBLY_PUSHDOWN_NPDA);
}

void xmlApi<automaton::VisiblyPushdownNPDA>::compose(std::list<sax::Token>& output, const automaton::VisiblyPushdownNPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::RealTimeHeightDeterministicDPDA xmlApi<automaton::RealTimeHeightDeterministicDPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseRealTimeHeightDeterministicDPDA(input);
}

bool xmlApi<automaton::RealTimeHeightDeterministicDPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_DPDA);
}

void xmlApi<automaton::RealTimeHeightDeterministicDPDA>::compose(std::list<sax::Token>& output, const automaton::RealTimeHeightDeterministicDPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::RealTimeHeightDeterministicNPDA xmlApi<automaton::RealTimeHeightDeterministicNPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseRealTimeHeightDeterministicNPDA(input);
}

bool xmlApi<automaton::RealTimeHeightDeterministicNPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_NPDA);
}

void xmlApi<automaton::RealTimeHeightDeterministicNPDA>::compose(std::list<sax::Token>& output, const automaton::RealTimeHeightDeterministicNPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::NPDA xmlApi<automaton::NPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseNPDA(input);
}

bool xmlApi<automaton::NPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_NPDA);
}

void xmlApi<automaton::NPDA>::compose(std::list<sax::Token>& output, const automaton::NPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::SinglePopNPDA xmlApi<automaton::SinglePopNPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseSinglePopNPDA(input);
}

bool xmlApi<automaton::SinglePopNPDA>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_SINGLE_POP_NPDA);
}

void xmlApi<automaton::SinglePopNPDA>::compose(std::list<sax::Token>& output, const automaton::SinglePopNPDA& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::OneTapeDTM xmlApi<automaton::OneTapeDTM>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseOneTapeDTM(input);
}

bool xmlApi<automaton::OneTapeDTM>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_ONE_TAPE_DTM);
}

void xmlApi<automaton::OneTapeDTM>::compose(std::list<sax::Token>& output, const automaton::OneTapeDTM& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}

automaton::State xmlApi<automaton::State>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseState(input);
}

bool xmlApi<automaton::State>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::AUTOMATON_STATE);
}

void xmlApi<automaton::State>::compose(std::list<sax::Token>& output, const automaton::State& data) {
	ToXMLComposers::automatonComposer.compose(output, data);
}


grammar::Grammar xmlApi<grammar::Grammar>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseGrammar(input);
}

bool xmlApi<grammar::Grammar>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.first(input);
}

void xmlApi<grammar::Grammar>::compose(std::list<sax::Token>& output, const grammar::Grammar& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::LeftLG xmlApi<grammar::LeftLG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseLeftLG(input);
}

bool xmlApi<grammar::LeftLG>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_LEFT_LG);
}

void xmlApi<grammar::LeftLG>::compose(std::list<sax::Token>& output, const grammar::LeftLG& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::LeftRG xmlApi<grammar::LeftRG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseLeftRG(input);
}

bool xmlApi<grammar::LeftRG>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_LEFT_RG);
}

void xmlApi<grammar::LeftRG>::compose(std::list<sax::Token>& output, const grammar::LeftRG& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::RightLG xmlApi<grammar::RightLG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseRightLG(input);
}

bool xmlApi<grammar::RightLG>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_RIGHT_LG);
}

void xmlApi<grammar::RightLG>::compose(std::list<sax::Token>& output, const grammar::RightLG& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::RightRG xmlApi<grammar::RightRG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseRightRG(input);
}

bool xmlApi<grammar::RightRG>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_RIGHT_RG);
}

void xmlApi<grammar::RightRG>::compose(std::list<sax::Token>& output, const grammar::RightRG& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::LG xmlApi<grammar::LG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseLG(input);
}

bool xmlApi<grammar::LG>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_LG);
}

void xmlApi<grammar::LG>::compose(std::list<sax::Token>& output, const grammar::LG& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::CFG xmlApi<grammar::CFG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseCFG(input);
}

bool xmlApi<grammar::CFG>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_CFG);
}

void xmlApi<grammar::CFG>::compose(std::list<sax::Token>& output, const grammar::CFG& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::EpsilonFreeCFG xmlApi<grammar::EpsilonFreeCFG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseEpsilonFreeCFG(input);
}

bool xmlApi<grammar::EpsilonFreeCFG>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_EPSILON_FREE_CFG);
}

void xmlApi<grammar::EpsilonFreeCFG>::compose(std::list<sax::Token>& output, const grammar::EpsilonFreeCFG& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::CNF xmlApi<grammar::CNF>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseCNF(input);
}

bool xmlApi<grammar::CNF>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_CNF);
}

void xmlApi<grammar::CNF>::compose(std::list<sax::Token>& output, const grammar::CNF& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::GNF xmlApi<grammar::GNF>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseGNF(input);
}

bool xmlApi<grammar::GNF>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_GNF);
}

void xmlApi<grammar::GNF>::compose(std::list<sax::Token>& output, const grammar::GNF& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::CSG xmlApi<grammar::CSG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseCSG(input);
}

bool xmlApi<grammar::CSG>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_CSG);
}

void xmlApi<grammar::CSG>::compose(std::list<sax::Token>& output, const grammar::CSG& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::NonContractingGrammar xmlApi<grammar::NonContractingGrammar>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseNonContractingGrammar(input);
}

bool xmlApi<grammar::NonContractingGrammar>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_NON_CONTRACTING_GRAMMAR);
}

void xmlApi<grammar::NonContractingGrammar>::compose(std::list<sax::Token>& output, const grammar::NonContractingGrammar& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::ContextPreservingUnrestrictedGrammar xmlApi<grammar::ContextPreservingUnrestrictedGrammar>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseContextPreservingUnrestrictedGrammar(input);
}

bool xmlApi<grammar::ContextPreservingUnrestrictedGrammar>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR);
}

void xmlApi<grammar::ContextPreservingUnrestrictedGrammar>::compose(std::list<sax::Token>& output, const grammar::ContextPreservingUnrestrictedGrammar& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}

grammar::UnrestrictedGrammar xmlApi<grammar::UnrestrictedGrammar>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseUnrestrictedGrammar(input);
}

bool xmlApi<grammar::UnrestrictedGrammar>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::GRAMMAR_UNRESTRICTED_GRAMMAR);
}

void xmlApi<grammar::UnrestrictedGrammar>::compose(std::list<sax::Token>& output, const grammar::UnrestrictedGrammar& data) {
	ToXMLComposers::grammarComposer.compose(output, data);
}


label::Label xmlApi<label::Label>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseLabel(input);
}

bool xmlApi<label::Label>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.first(input);
}

void xmlApi<label::Label>::compose(std::list<sax::Token>& output, const label::Label& data) {
	ToXMLComposers::labelComposer.compose(output, data);
}

label::PrimitiveLabel xmlApi<label::PrimitiveLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parsePrimitiveLabel(input);
}

bool xmlApi<label::PrimitiveLabel>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::LABEL_PRIMITIVE_LABEL);
}

void xmlApi<label::PrimitiveLabel>::compose(std::list<sax::Token>& output, const label::PrimitiveLabel& data) {
	ToXMLComposers::labelComposer.compose(output, data);
}

label::HexavigesimalLabel xmlApi<label::HexavigesimalLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseHexavigesimalLabel(input);
}

bool xmlApi<label::HexavigesimalLabel>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::LABEL_PRIMITIVE_LABEL);
}

void xmlApi<label::HexavigesimalLabel>::compose(std::list<sax::Token>& output, const label::HexavigesimalLabel& data) {
	ToXMLComposers::labelComposer.compose(output, data);
}

label::ObjectLabel xmlApi<label::ObjectLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseObjectLabel(input);
}

bool xmlApi<label::ObjectLabel>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::LABEL_OBJECT_LABEL);
}

void xmlApi<label::ObjectLabel>::compose(std::list<sax::Token>& output, const label::ObjectLabel& data) {
	ToXMLComposers::labelComposer.compose(output, data);
}

label::LabelSetLabel xmlApi<label::LabelSetLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseLabelSetLabel(input);
}

bool xmlApi<label::LabelSetLabel>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::LABEL_LABEL_SET_LABEL);
}

void xmlApi<label::LabelSetLabel>::compose(std::list<sax::Token>& output, const label::LabelSetLabel& data) {
	ToXMLComposers::labelComposer.compose(output, data);
}

label::LabelPairLabel xmlApi<label::LabelPairLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseLabelPairLabel(input);
}

bool xmlApi<label::LabelPairLabel>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::LABEL_LABEL_PAIR_LABEL);
}

void xmlApi<label::LabelPairLabel>::compose(std::list<sax::Token>& output, const label::LabelPairLabel& data) {
	ToXMLComposers::labelComposer.compose(output, data);
}

label::UniqueLabel xmlApi<label::UniqueLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseUniqueLabel(input);
}

bool xmlApi<label::UniqueLabel>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::LABEL_UNIQUE_LABEL);
}

void xmlApi<label::UniqueLabel>::compose(std::list<sax::Token>& output, const label::UniqueLabel& data) {
	ToXMLComposers::labelComposer.compose(output, data);
}


regexp::RegExp xmlApi<regexp::RegExp>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::regexpParser.parseRegExp(input);
}

bool xmlApi<regexp::RegExp>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::regexpParser.first(input);
}

void xmlApi<regexp::RegExp>::compose(std::list<sax::Token>& output, const regexp::RegExp& data) {
	ToXMLComposers::regexpComposer.compose(output, data);
}

regexp::UnboundedRegExp xmlApi<regexp::UnboundedRegExp>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::regexpParser.parseUnboundedRegExp(input);
}

bool xmlApi<regexp::UnboundedRegExp>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::REGEXP_UNBOUNDED_REGEXP);
}

void xmlApi<regexp::UnboundedRegExp>::compose(std::list<sax::Token>& output, const regexp::UnboundedRegExp& data) {
	ToXMLComposers::regexpComposer.compose(output, data);
}

regexp::FormalRegExp xmlApi<regexp::FormalRegExp>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::regexpParser.parseFormalRegExp(input);
}

bool xmlApi<regexp::FormalRegExp>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::REGEXP_FORMAL_REGEXP);
}

void xmlApi<regexp::FormalRegExp>::compose(std::list<sax::Token>& output, const regexp::FormalRegExp& data) {
	ToXMLComposers::regexpComposer.compose(output, data);
}


string::String xmlApi<string::String>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.parseString(input);
}

bool xmlApi<string::String>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.first(input);
}

void xmlApi<string::String>::compose(std::list<sax::Token>& output, const string::String& data) {
	ToXMLComposers::stringComposer.compose(output, data);
}

string::LinearString xmlApi<string::LinearString>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.parseLinearString(input);
}

bool xmlApi<string::LinearString>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::STRING_LINEAR_STRING);
}

void xmlApi<string::LinearString>::compose(std::list<sax::Token>& output, const string::LinearString& data) {
	ToXMLComposers::stringComposer.compose(output, data);
}

string::CyclicString xmlApi<string::CyclicString>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.parseCyclicString(input);
}

bool xmlApi<string::CyclicString>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::STRING_CYCLIC_STRING);
}

void xmlApi<string::CyclicString>::compose(std::list<sax::Token>& output, const string::CyclicString& data) {
	ToXMLComposers::stringComposer.compose(output, data);
}

string::Epsilon xmlApi<string::Epsilon>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.parseEpsilon(input);
}

bool xmlApi<string::Epsilon>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::STRING_EPSILON);
}

void xmlApi<string::Epsilon>::compose(std::list<sax::Token>& output, const string::Epsilon& data) {
	ToXMLComposers::stringComposer.compose(output, data);
}


primitive::Primitive xmlApi<primitive::Primitive>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parsePrimitive(input);
}

bool xmlApi<primitive::Primitive>::first(const std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.first(input);
}

void xmlApi<primitive::Primitive>::compose(std::list<sax::Token>& output, const primitive::Primitive& data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

primitive::String xmlApi<primitive::String>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseString(input);
}

bool xmlApi<primitive::String>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_STRING);
}

void xmlApi<primitive::String>::compose(std::list<sax::Token>& output, const primitive::String& data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

std::string xmlApi<std::string>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseStringRaw(input);
}

bool xmlApi<std::string>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_STRING);
}

void xmlApi<std::string>::compose(std::list<sax::Token>& output, const std::string& data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

primitive::Integer xmlApi<primitive::Integer>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseInteger(input);
}

bool xmlApi<primitive::Integer>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_INTEGER);
}

void xmlApi<primitive::Integer>::compose(std::list<sax::Token>& output, const primitive::Integer& data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

int xmlApi<int>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseIntegerRaw(input);
}

bool xmlApi<int>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_INTEGER);
}

void xmlApi<int>::compose(std::list<sax::Token>& output, int data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

primitive::Character xmlApi<primitive::Character>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseCharacter(input);
}

bool xmlApi<primitive::Character>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_CHARACTER);
}

void xmlApi<primitive::Character>::compose(std::list<sax::Token>& output, const primitive::Character& data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

primitive::Unsigned xmlApi<primitive::Unsigned>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseUnsigned(input);
}

bool xmlApi<primitive::Unsigned>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_UNSIGNED);
}

void xmlApi<primitive::Unsigned>::compose(std::list<sax::Token>& output, const primitive::Unsigned& data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

primitive::Bool xmlApi<primitive::Bool>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseBool(input);
}

bool xmlApi<primitive::Bool>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_BOOL);
}

void xmlApi<primitive::Bool>::compose(std::list<sax::Token>& output, const primitive::Bool& data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

char xmlApi<char>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseCharacterRaw(input);
}

bool xmlApi<char>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_CHARACTER);
}

void xmlApi<char>::compose(std::list<sax::Token>& output, char data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

unsigned xmlApi<unsigned>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseCharacterRaw(input);
}

bool xmlApi<unsigned>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_CHARACTER);
}

void xmlApi<unsigned>::compose(std::list<sax::Token>& output, unsigned data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

bool xmlApi<bool>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseCharacterRaw(input);
}

bool xmlApi<bool>::first(const std::list<sax::Token>& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, Names::PRIMITIVE_CHARACTER);
}

void xmlApi<bool>::compose(std::list<sax::Token>& output, bool data) {
	ToXMLComposers::primitiveComposer.compose(output, data);
}

void ToXMLComposers::Visit(void* data, const Void& voidObject) const {
	xmlApi<Void>::compose(*((std::list<sax::Token>*) data), voidObject);
}

void ToXMLComposers::Visit(void* data, const container::ObjectsSet& container) const {
	xmlApi<container::ObjectsSet>::compose(*((std::list<sax::Token>*) data), container);
}

void ToXMLComposers::Visit(void* data, const container::ObjectsVector& container) const {
	xmlApi<container::ObjectsVector>::compose(*((std::list<sax::Token>*) data), container);
}

void ToXMLComposers::Visit(void* data, const container::ObjectsPair& container) const {
	xmlApi<container::ObjectsPair>::compose(*((std::list<sax::Token>*) data), container);
}

void ToXMLComposers::Visit(void* data, const container::ObjectsMap& container) const {
	xmlApi<container::ObjectsMap>::compose(*((std::list<sax::Token>*) data), container);
}

void ToXMLComposers::Visit(void* data, const exception::AlibException& symbol) const {
	xmlApi<exception::AlibException>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::BlankSymbol& symbol) const {
	xmlApi<alphabet::BlankSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::BottomOfTheStackSymbol& symbol) const {
	xmlApi<alphabet::BottomOfTheStackSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::EndSymbol& symbol) const {
	xmlApi<alphabet::EndSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::StartSymbol& symbol) const {
	xmlApi<alphabet::StartSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::RankedSymbol& symbol) const {
	xmlApi<alphabet::RankedSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::BarSymbol& symbol) const {
	xmlApi<alphabet::BarSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::RankedBarSymbol& symbol) const {
	xmlApi<alphabet::RankedBarSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::SubtreeWildcardSymbol& symbol) const {
	xmlApi<alphabet::SubtreeWildcardSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::LabeledSymbol& symbol) const {
	xmlApi<alphabet::LabeledSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::SymbolSetSymbol& symbol) const {
	xmlApi<alphabet::SymbolSetSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::SymbolPairSymbol& symbol) const {
	xmlApi<alphabet::SymbolPairSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const alphabet::UniqueSymbol& symbol) const {
	xmlApi<alphabet::UniqueSymbol>::compose(*((std::list<sax::Token>*) data), symbol);
}

void ToXMLComposers::Visit(void* data, const automaton::EpsilonNFA& automaton) const {
	xmlApi<automaton::EpsilonNFA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::MultiInitialStateNFA& automaton) const {
	xmlApi<automaton::MultiInitialStateNFA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::NFA& automaton) const {
	xmlApi<automaton::NFA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::DFA& automaton) const {
	xmlApi<automaton::DFA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::ExtendedNFA& automaton) const {
	xmlApi<automaton::ExtendedNFA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::CompactNFA& automaton) const {
	xmlApi<automaton::CompactNFA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::DPDA& automaton) const {
	xmlApi<automaton::DPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::SinglePopDPDA& automaton) const {
	xmlApi<automaton::SinglePopDPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::InputDrivenNPDA& automaton) const {
	xmlApi<automaton::InputDrivenNPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::VisiblyPushdownDPDA& automaton) const {
	xmlApi<automaton::VisiblyPushdownDPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::VisiblyPushdownNPDA& automaton) const {
	xmlApi<automaton::VisiblyPushdownNPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::RealTimeHeightDeterministicDPDA& automaton) const {
	xmlApi<automaton::RealTimeHeightDeterministicDPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::RealTimeHeightDeterministicNPDA& automaton) const {
	xmlApi<automaton::RealTimeHeightDeterministicNPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::NPDA& automaton) const {
	xmlApi<automaton::NPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::SinglePopNPDA& automaton) const {
	xmlApi<automaton::SinglePopNPDA>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const automaton::OneTapeDTM& automaton) const {
	xmlApi<automaton::OneTapeDTM>::compose(*((std::list<sax::Token>*) data), automaton);
}

void ToXMLComposers::Visit(void* data, const grammar::LeftLG& grammar) const {
	xmlApi<grammar::LeftLG>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::LeftRG& grammar) const {
	xmlApi<grammar::LeftRG>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::RightLG& grammar) const {
	xmlApi<grammar::RightLG>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::RightRG& grammar) const {
	xmlApi<grammar::RightRG>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::LG& grammar) const {
	xmlApi<grammar::LG>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::CFG& grammar) const {
	xmlApi<grammar::CFG>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	xmlApi<grammar::EpsilonFreeCFG>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::CNF& grammar) const {
	xmlApi<grammar::CNF>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::GNF& grammar) const {
	xmlApi<grammar::GNF>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::CSG& grammar) const {
	xmlApi<grammar::CSG>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::NonContractingGrammar& grammar) const {
	xmlApi<grammar::NonContractingGrammar>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const {
	xmlApi<grammar::ContextPreservingUnrestrictedGrammar>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const grammar::UnrestrictedGrammar& grammar) const {
	xmlApi<grammar::UnrestrictedGrammar>::compose(*((std::list<sax::Token>*) data), grammar);
}

void ToXMLComposers::Visit(void* data, const label::PrimitiveLabel& label) const {
	xmlApi<label::PrimitiveLabel>::compose(*((std::list<sax::Token>*) data), label);
}

void ToXMLComposers::Visit(void* data, const label::HexavigesimalLabel& label) const {
	xmlApi<label::HexavigesimalLabel>::compose(*((std::list<sax::Token>*) data), label);
}

void ToXMLComposers::Visit(void* data, const label::ObjectLabel& label) const {
	xmlApi<label::ObjectLabel>::compose(*((std::list<sax::Token>*) data), label);
}

void ToXMLComposers::Visit(void* data, const label::LabelSetLabel& label) const {
	xmlApi<label::LabelSetLabel>::compose(*((std::list<sax::Token>*) data), label);
}

void ToXMLComposers::Visit(void* data, const label::LabelPairLabel& label) const {
	xmlApi<label::LabelPairLabel>::compose(*((std::list<sax::Token>*) data), label);
}

void ToXMLComposers::Visit(void* data, const label::UniqueLabel& label) const {
	xmlApi<label::UniqueLabel>::compose(*((std::list<sax::Token>*) data), label);
}

void ToXMLComposers::Visit(void* data, const regexp::UnboundedRegExp& regexp) const {
	xmlApi<regexp::UnboundedRegExp>::compose(*((std::list<sax::Token>*) data), regexp);
}

void ToXMLComposers::Visit(void* data, const regexp::FormalRegExp& regexp) const {
	xmlApi<regexp::FormalRegExp>::compose(*((std::list<sax::Token>*) data), regexp);
}

void ToXMLComposers::Visit(void* data, const string::Epsilon& string) const {
	xmlApi<string::Epsilon>::compose(*((std::list<sax::Token>*) data), string);
}

void ToXMLComposers::Visit(void* data, const string::CyclicString& string) const {
	xmlApi<string::CyclicString>::compose(*((std::list<sax::Token>*) data), string);
}

void ToXMLComposers::Visit(void* data, const string::LinearString& string) const {
	xmlApi<string::LinearString>::compose(*((std::list<sax::Token>*) data), string);
}

void ToXMLComposers::Visit(void* data, const primitive::String& primitive) const {
	xmlApi<primitive::String>::compose(*((std::list<sax::Token>*) data), primitive);
}

void ToXMLComposers::Visit(void* data, const primitive::Integer& primitive) const {
	xmlApi<primitive::Integer>::compose(*((std::list<sax::Token>*) data), primitive);
}

void ToXMLComposers::Visit(void* data, const primitive::Character& primitive) const {
	xmlApi<primitive::Character>::compose(*((std::list<sax::Token>*) data), primitive);
}

void ToXMLComposers::Visit(void* data, const primitive::Unsigned& primitive) const {
	xmlApi<primitive::Unsigned>::compose(*((std::list<sax::Token>*) data), primitive);
}

void ToXMLComposers::Visit(void* data, const primitive::Bool& primitive) const {
	xmlApi<primitive::Bool>::compose(*((std::list<sax::Token>*) data), primitive);
}

} /* namespace alib */
