/*
 * BlankSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef BLANK_SYMBOL_H_
#define BLANK_SYMBOL_H_

#include "Symbol.h"

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class BlankSymbol : public std::acceptor<BlankSymbol, VisitableSymbolBase, std::acceptor<BlankSymbol, alib::VisitableObjectBase, SymbolBase> > {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit BlankSymbol();

	virtual SymbolBase* clone() const;
	virtual SymbolBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const BlankSymbol& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<BlankSymbol>();
	}

	static BlankSymbol BLANK;
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::BlankSymbol> {
	int operator()(const alphabet::BlankSymbol& first, const alphabet::BlankSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* BLANK_SYMBOL_H_ */
