/*
 * RankedSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef RANKED_SYMBOL_H_
#define RANKED_SYMBOL_H_

#include "../label/Label.h"
#include "SymbolBase.h"
#include <ostream>
#include "../primitive/Integer.h"

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class RankedSymbol : public std::acceptor<RankedSymbol, VisitableSymbolBase, std::acceptor<RankedSymbol, alib::VisitableObjectBase, SymbolBase> > {
protected:
	label::Label label;
	primitive::Integer rank;
	
public:
	explicit RankedSymbol(int number, int rank);
	explicit RankedSymbol(char character, int rank);
	explicit RankedSymbol(const std::string& label, int rank);

	/**
	 * Creates new symbol with given name and rank.
	 * @param symbol name of the symbol
	 * @param rank of the symbol
	 */
	explicit RankedSymbol(const label::Label& label, const primitive::Integer& rank);
	explicit RankedSymbol(label::Label&& label, primitive::Integer&& rank);

	virtual SymbolBase* clone() const;
	virtual SymbolBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const label::Label& getLabel() const;

	/**
	 * @return rank of the symbol
	 */
	const primitive::Integer& getRank() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const RankedSymbol& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<RankedSymbol>();
	}
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::RankedSymbol> {
	int operator()(const alphabet::RankedSymbol& first, const alphabet::RankedSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* RANKED_SYMBOL_H_ */
