/*
 * SymbolSetSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "SymbolSetSymbol.h"
#include "../std/set.hpp"
#include <sstream>

namespace alphabet {

SymbolSetSymbol::SymbolSetSymbol(const std::set<Symbol>& symbol) : symbol(symbol) {

}

SymbolSetSymbol::SymbolSetSymbol(std::set<Symbol>&& symbol) : symbol(std::move(symbol)) {

}

SymbolBase* SymbolSetSymbol::clone() const {
	return new SymbolSetSymbol(*this);
}

SymbolBase* SymbolSetSymbol::plunder() && {
	return new SymbolSetSymbol(std::move(*this));
}

const std::set<Symbol>& SymbolSetSymbol::getData() const {
	return symbol;
}

int SymbolSetSymbol::compare(const SymbolSetSymbol& other) const {
	std::compare<std::set<Symbol>> comp;
	return comp(symbol, other.symbol);
}

void SymbolSetSymbol::operator>>(std::ostream& out) const {
	out << "(SymbolSetSymbol " << symbol << ")";
}

SymbolSetSymbol::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	bool first = true;
	for(const Symbol& subSymbol : symbol) {
		if(!first) 
			ss << ", ";
		else
			first = false;
		ss << (std::string) subSymbol;
	}
	ss << "}";
	return std::move(ss).str();
}

} /* namespace alphabet */

