/*
 * SymbolSetSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_SET_SYMBOL_H_
#define SYMBOL_SET_SYMBOL_H_

#include <string>
#include <ostream>

#include "Symbol.h"
#include "SymbolBase.h"
#include <set>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class SymbolSetSymbol : public std::acceptor<SymbolSetSymbol, VisitableSymbolBase, std::acceptor<SymbolSetSymbol, alib::VisitableObjectBase, SymbolBase> > {
protected:
	std::set<Symbol> symbol;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit SymbolSetSymbol(const std::set<Symbol>& symbol);

	explicit SymbolSetSymbol(std::set<Symbol>&& symbol);

	virtual SymbolBase* clone() const;

	virtual SymbolBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const std::set<Symbol>& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const SymbolSetSymbol& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<SymbolSetSymbol>();
	}
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::SymbolSetSymbol> {
	int operator()(const alphabet::SymbolSetSymbol& first, const alphabet::SymbolSetSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* SYMBOL_SET_SYMBOL_H_ */

