/*
 * UniqueSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueSymbol.h"
#include "../std/set.hpp"
#include <sstream>

namespace alphabet {

UniqueSymbol::UniqueSymbol(const Symbol& symbol, const primitive::Integer& id) : symbol(symbol), id(id) {

}

SymbolBase* UniqueSymbol::clone() const {
	return new UniqueSymbol(*this);
}

SymbolBase* UniqueSymbol::plunder() && {
	return new UniqueSymbol(std::move(*this));
}

const Symbol& UniqueSymbol::getSymbol() const {
	return symbol;
}

const primitive::Integer& UniqueSymbol::getId() const {
	return id;
}

int UniqueSymbol::compare(const UniqueSymbol& other) const {
	int res = symbol.getData().compare(other.symbol.getData());
	if(res == 0) res = id.compare(other.id);
	return res;
}

void UniqueSymbol::operator>>(std::ostream& out) const {
	out << "(UniqueSymbol " << symbol << ", " << id << ")";
}

UniqueSymbol::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << symbol;
	ss << ", ";
	ss << id;
	ss << "}";
	return std::move(ss).str();
}

} /* namespace alphabet */

