/*
 * BlankSymbolInputTapeAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "BlankSymbolInputTapeAlphabet.h"

#include <algorithm>
#include "../AutomatonException.h"

namespace automaton {

BlankSymbolInputTapeAlphabet::BlankSymbolInputTapeAlphabet(const alphabet::Symbol& blank) : blankSymbol(blank) {
	tapeAlphabet.insert(blank);
}

const std::set<alphabet::Symbol>& BlankSymbolInputTapeAlphabet::getInputAlphabet() const {
	return inputAlphabet;
}

bool BlankSymbolInputTapeAlphabet::addInputSymbol(const alphabet::Symbol& symbol) {
	if (tapeAlphabet.find(symbol) == tapeAlphabet.end()) {
		throw AutomatonException("Input symbol \"" + (std::string) symbol + "\" is not in tape alphabet.");
	}

	if (symbol == blankSymbol) {
		throw AutomatonException("Input symbol \"" + (std::string) symbol + "\" cannot be blank symbol.");
	}

	return inputAlphabet.insert(symbol).second;
}

void BlankSymbolInputTapeAlphabet::setInputAlphabet(const std::set<alphabet::Symbol>& newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(inputAlphabet.begin(), inputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(newSymbols.begin(), newSymbols.end(), inputAlphabet.begin(), inputAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addInputSymbol(addedSymbol);
	}
}

bool BlankSymbolInputTapeAlphabet::addTapeSymbol(const alphabet::Symbol& symbol) {
	return tapeAlphabet.insert(symbol).second;
}

void BlankSymbolInputTapeAlphabet::setTapeAlphabet(const std::set<alphabet::Symbol>& newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(tapeAlphabet.begin(), tapeAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(newSymbols.begin(), newSymbols.end(), tapeAlphabet.begin(), tapeAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeTapeSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addTapeSymbol(addedSymbol);
	}
}

const std::set<alphabet::Symbol>& BlankSymbolInputTapeAlphabet::getTapeAlphabet() const {
	return tapeAlphabet;
}

void BlankSymbolInputTapeAlphabet::setBlankSymbol(const alphabet::Symbol& symbol) {
	if (inputAlphabet.find(symbol) != inputAlphabet.end())
		throw AutomatonException("Blank symbol \"" + (std::string) symbol + "\" is in input alphabet.");

	if (tapeAlphabet.find(symbol) == tapeAlphabet.end())
		throw AutomatonException("Blank symbol \"" + (std::string) symbol + "\" is not in tape alphabet.");

	blankSymbol = symbol;

}

const alphabet::Symbol& BlankSymbolInputTapeAlphabet::getBlankSymbol() const {
	return blankSymbol;
}

} /* namespace automaton */

