/*
 * State.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#ifndef STATE_H_
#define STATE_H_

#include <string>
#include <ostream>

#include "../../label/Label.h"

namespace automaton {

/**
 * Class representing state in an automaton.
 */
class State {
private:
	label::Label name;
public:
	explicit State(const label::Label& name);
	explicit State(label::Label&& name);
	explicit State(int number);
	explicit State(char character);
	explicit State(const std::string& name);

	const label::Label& getName() const;

	bool operator < (const State& other) const;
	bool operator == (const State& other) const;
	bool operator != (const State& other) const;
	int compare(const State& other) const;

	friend std::ostream& operator<<(std::ostream&, const State&);

	operator std::string () const;
};

} /* namespace automaton */

namespace std {

template<>
struct compare<automaton::State> {
	int operator()(const automaton::State& first, const automaton::State& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* STATE_H_ */
