/*
 * ContainerFromXMLParser.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "ContainerFromXMLParser.hpp"
#include "../sax/ParserException.h"

namespace container {

Container ContainerFromXMLParser::parseContainer(std::list<sax::Token>& input) const {
	return parseContainer(input, std::set<FEATURES>({FEATURES::SET, FEATURES::VECTOR, FEATURES::PAIR, FEATURES::MAP}));
}

Container ContainerFromXMLParser::parseContainer(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(alib::xmlApi<ObjectsSet>::first(input)) {
		if(!features.count(FEATURES::SET)) throw exception::AlibException();
		return Container(parseObjectsSet(input));
	} else if(alib::xmlApi<ObjectsVector>::first(input)) {
		if(!features.count(FEATURES::VECTOR)) throw exception::AlibException();
		return Container(parseObjectsVector(input));
	} else if(alib::xmlApi<ObjectsPair>::first(input)) {
		if(!features.count(FEATURES::PAIR)) throw exception::AlibException();
		return Container(parseObjectsPair(input));
	} else if(alib::xmlApi<ObjectsMap>::first(input)) {
		if(!features.count(FEATURES::MAP)) throw exception::AlibException();
		return Container(parseObjectsMap(input));
	} else {
		throw sax::ParserException(sax::Token("set / vector / pair / map", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

bool ContainerFromXMLParser::first(const std::list<sax::Token>& input) const {
	if(alib::xmlApi<ObjectsSet>::first(input) || alib::xmlApi<ObjectsVector>::first(input) || alib::xmlApi<ObjectsPair>::first(input) || alib::xmlApi<ObjectsMap>::first(input)) {
		return true;
	} else {
		return false;
	}
}

ObjectsSet ContainerFromXMLParser::parseObjectsSet(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::CONTAINER_OBJECTS_SET);

	ObjectsSet set;

	while(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		set.insert(alib::xmlApi<alib::Object>::parse(input));
	}

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::CONTAINER_OBJECTS_SET);
	return set;
}

ObjectsVector ContainerFromXMLParser::parseObjectsVector(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::CONTAINER_OBJECTS_VECTOR);

	ObjectsVector vector;

	while(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		vector.push_back(alib::xmlApi<alib::Object>::parse(input));
	}

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::CONTAINER_OBJECTS_VECTOR);
	return vector;
}

ObjectsPair ContainerFromXMLParser::parseObjectsPair(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::CONTAINER_OBJECTS_PAIR);

	alib::Object first = alib::xmlApi<alib::Object>::parse(input);
	alib::Object second = alib::xmlApi<alib::Object>::parse(input);

	ObjectsPair pair(first, second);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::CONTAINER_OBJECTS_PAIR);
	return pair;
}

ObjectsMap ContainerFromXMLParser::parseObjectsMap(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::CONTAINER_OBJECTS_MAP);

	ObjectsMap map;
	while(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		map.insert(alib::xmlApi<std::pair<alib::Object, alib::Object>>::parse(input));
	}

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::CONTAINER_OBJECTS_MAP);
	return map;
}

} /* namespace container */

