/*
 * Map.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_MAP_H_
#define OBJECTS_MAP_H_

#include <map>
#include <string>

#include "../object/Object.h"
#include "ContainerBase.h"

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsMap : public std::map<alib::Object, alib::Object>, public std::acceptor<ObjectsMap, VisitableContainerBase, std::acceptor<ObjectsMap, alib::VisitableObjectBase, ContainerBase> > {
public:
	virtual ContainerBase* clone() const;
	
	virtual ContainerBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const ObjectsMap& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<ObjectsMap>();
	}
};

} /* namespace container */

namespace std {

template<>
struct compare<container::ObjectsMap> {
	int operator()(const container::ObjectsMap& first, const container::ObjectsMap& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* OBJECTS_MAP_H_ */
