/*
 * Set.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_VECTOR_H_
#define OBJECTS_VECTOR_H_

#include <vector>
#include <string>

#include "../object/Object.h"
#include "ContainerBase.h"

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsVector : public std::vector<alib::Object>, public std::acceptor<ObjectsVector, VisitableContainerBase, std::acceptor<ObjectsVector, alib::VisitableObjectBase, ContainerBase> > {
public:
	virtual ContainerBase* clone() const;
	
	virtual ContainerBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const ObjectsVector& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<ObjectsVector>();
	}
};

} /* namespace container */

namespace std {

template<>
struct compare<container::ObjectsVector> {
	int operator()(const container::ObjectsVector& first, const container::ObjectsVector& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* OBJECTS_VECTOR_H_ */
