/*
 * ExceptionFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "ExceptionFromXMLParser.h"
#include "../sax/ParserException.h"
#include <string>

#include "../XmlApi.hpp"

namespace exception {

AlibException ExceptionFromXMLParser::parseException(std::list<sax::Token>& input) const {
	return parseException(input, std::set<FEATURES>({FEATURES::ALIB_EXCEPTION}));
}

AlibException ExceptionFromXMLParser::parseException(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(!features.count(FEATURES::ALIB_EXCEPTION)) throw exception::AlibException();
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::EXCEPTION_ALIB_EXCEPTION);
	popToken(input, sax::Token::TokenType::START_ELEMENT, "cause");
	std::string cause = "";
	if (input.front().getType() == sax::Token::TokenType::CHARACTER) {
		cause = input.front().getData();
		input.pop_front();
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "cause");
	popToken(input, sax::Token::TokenType::START_ELEMENT, "backtrace");
	std::string backtrace = "";
	if (input.front().getType() == sax::Token::TokenType::CHARACTER) {
		backtrace = input.front().getData();
		input.pop_front();
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "backtrace");
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::EXCEPTION_ALIB_EXCEPTION);
	return AlibException(cause, backtrace);
}

bool ExceptionFromXMLParser::first(const std::list<sax::Token>& input) const {
	if(alib::xmlApi<AlibException>::first(input)) {
		return true;
	} else {
		return false;
	}
}

} /* namespace exception */
