/*
 * GrammarFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarFromStringLexer.h"
#include "../std/istream.h"

namespace grammar {

GrammarFromStringLexer::Token GrammarFromStringLexer::next(std::istream& in) const {
	GrammarFromStringLexer::Token token;
	token.type = TokenType::ERROR;
	token.value = "";
	token.raw = "";
	char character;

L0:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == ' ' || character == '\t' || character == '\n') {
		token.raw += character;
		goto L0;
	} else if(character == '|') {
		token.type = TokenType::SEPARATOR;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '{') {
		token.type = TokenType::SET_BEGIN;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '}') {
		token.type = TokenType::SET_END;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '(') {
		token.type = TokenType::TUPLE_BEGIN;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == ')') {
		token.type = TokenType::TUPLE_END;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == ',') {
		token.type = TokenType::COMMA;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '-') {
		token.value += character;
		token.raw += character;
		goto L2;
	} else if(character == '#') {
		token.value += character;
		token.raw += character;
		goto L1;
	} else if(in.unget(), in >> "RIGHT_RG") {
		token.type = TokenType::RIGHT_RG;
		token.value = "RIGHT_RG";
		token.raw = "RIGHT_RG";
		return token;
	} else if(in.clear(), in >> "CNF") {
		token.type = TokenType::CNF;
		token.value = "CNF";
		token.raw = "CNF";
		return token;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
L1:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == 'E') {
		token.value += character;
		token.raw += character;
		token.type = TokenType::EPSILON;
		return token;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
L2:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == '>') {
		token.value += character;
		token.raw += character;
		token.type = TokenType::MAPS_TO;
		return token;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
}

void GrammarFromStringLexer::putback(std::istream& in, GrammarFromStringLexer::Token token) const {
	while(!token.raw.empty()) {
		in.putback(token.raw.back());
		token.raw.pop_back();
	}
	in.clear();
}

} /* namespace grammar */
