/*
 * GrammarToXMLComposer.cpp
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarToXMLComposer.h"
#include "../alphabet/BlankSymbol.h"

#include "../XmlApi.hpp"

namespace grammar {

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const GrammarBase& grammar) const {
	grammar.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const Grammar& grammar) const {
	grammar.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const LeftLG& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_LEFT_LG, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token(alib::Names::GRAMMAR_LEFT_LG, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const LeftRG& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_LEFT_RG, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token(alib::Names::GRAMMAR_LEFT_RG, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const RightLG& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_RIGHT_LG, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token(alib::Names::GRAMMAR_RIGHT_LG, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const RightRG& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_RIGHT_RG, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token(alib::Names::GRAMMAR_RIGHT_RG, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const LG& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_LG, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token(alib::Names::GRAMMAR_LG, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const CFG& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_CFG, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token(alib::Names::GRAMMAR_CFG, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const EpsilonFreeCFG& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_EPSILON_FREE_CFG, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token(alib::Names::GRAMMAR_EPSILON_FREE_CFG, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const CNF& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_CNF, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token(alib::Names::GRAMMAR_CNF, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const GNF& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_GNF, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token(alib::Names::GRAMMAR_GNF, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const CSG& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_CSG, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token(alib::Names::GRAMMAR_CSG, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const NonContractingGrammar& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_NON_CONTRACTING_GRAMMAR, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token(alib::Names::GRAMMAR_NON_CONTRACTING_GRAMMAR, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const ContextPreservingUnrestrictedGrammar& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token(alib::Names::GRAMMAR_CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::compose(std::list<sax::Token>& out, const UnrestrictedGrammar& grammar) const {
	out.push_back(sax::Token(alib::Names::GRAMMAR_UNRESTRICTED_GRAMMAR, sax::Token::TokenType::START_ELEMENT));

	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token(alib::Names::GRAMMAR_UNRESTRICTED_GRAMMAR, sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeNonterminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("nonterminalAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("nonterminalAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeTerminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("terminalAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("terminalAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeInitialSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("initialSymbol", sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.push_back(sax::Token("initialSymbol", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeGeneratesEpsilon(std::list<sax::Token>& out, bool generatesEpsilon) const {
	out.push_back(sax::Token("generatesEpsilon", sax::Token::TokenType::START_ELEMENT));
	if(generatesEpsilon) {
		out.push_back(sax::Token("true", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("true", sax::Token::TokenType::END_ELEMENT));
	} else {
		out.push_back(sax::Token("false", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("false", sax::Token::TokenType::END_ELEMENT));
	}
	out.push_back(sax::Token("generatesEpsilon", sax::Token::TokenType::END_ELEMENT));
}


void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const UnrestrictedGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLHS(out, rule.first);
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const ContextPreservingUnrestrictedGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLContext(out, std::get<0>(rule.first));
			composeRuleSingleSymbolLHS(out, std::get<1>(rule.first));
			composeRuleRContext(out, std::get<2>(rule.first));
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const NonContractingGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLHS(out, rule.first);
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const CSG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLContext(out, std::get<0>(rule.first));
			composeRuleSingleSymbolLHS(out, std::get<1>(rule.first));
			composeRuleRContext(out, std::get<2>(rule.first));
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const GNF& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleGNFRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const CNF& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleOneOrTwoSymbolsRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const EpsilonFreeCFG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const CFG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const LG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleLGRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const RightLG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleRightLGRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const RightRG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleOneOrTwoSymbolsRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const LeftLG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleLeftLGRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const LeftRG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleOneOrTwoSymbolsRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLContext(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("lContext", sax::Token::TokenType::START_ELEMENT));
	if(symbols.size() == 0) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("lContext", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("lhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.size() == 0) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("lhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleRContext(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("rContext", sax::Token::TokenType::START_ELEMENT));
	if(symbols.size() == 0) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("rContext", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleSingleSymbolLHS(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("lhs", sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.push_back(sax::Token("lhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleRHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.size() == 0) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleOneOrTwoSymbolsRHS(std::list<sax::Token>& out, const std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.is<alphabet::Symbol>()) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbols.get<alphabet::Symbol>());
	} else {
		const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = symbols.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();
		alib::xmlApi<alphabet::Symbol>::compose(out, rhs.first);
		alib::xmlApi<alphabet::Symbol>::compose(out, rhs.second);
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleGNFRHS(std::list<sax::Token>& out, const std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));

	alib::xmlApi<alphabet::Symbol>::compose(out, symbols.first);

	for (const auto& symbol : symbols.second) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}

	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.is<std::vector<alphabet::Symbol>>()) {
		const std::vector<alphabet::Symbol>& rhs = symbols.get<std::vector<alphabet::Symbol>>();
		if(rhs.size() == 0) {
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
		} else for (const auto& symbol : symbols.get<std::vector<alphabet::Symbol>>()) {
			alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
		}
	} else {
		const std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>& rhs = symbols.get<std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>();
		for (const auto& symbol : std::get<0>(rhs)) {
			alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
		}
		alib::xmlApi<alphabet::Symbol>::compose(out, std::get<1>(rhs));
		for (const auto& symbol : std::get<2>(rhs)) {
			alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
		}
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLeftLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.is<std::vector<alphabet::Symbol>>()) {
		const std::vector<alphabet::Symbol>& rhs = symbols.get<std::vector<alphabet::Symbol>>();
		if(rhs.size() == 0) {
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
		} else for (const auto& symbol : symbols.get<std::vector<alphabet::Symbol>>()) {
			alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
		}
	} else {
		const std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>& rhs = symbols.get<std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>>();
		alib::xmlApi<alphabet::Symbol>::compose(out, rhs.first);
		for (const auto& symbol : rhs.second) {
			alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
		}
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleRightLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.is<std::vector<alphabet::Symbol>>()) {
		const std::vector<alphabet::Symbol>& rhs = symbols.get<std::vector<alphabet::Symbol>>();
		if(rhs.size() == 0) {
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
		} else {
			for (const auto& symbol : symbols.get<std::vector<alphabet::Symbol>>()) {
				alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
			}
		}
	} else {
		const std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>& rhs = symbols.get<std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>>();
		for (const auto& symbol : rhs.first) {
			alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
		}
		alib::xmlApi<alphabet::Symbol>::compose(out, rhs.second);
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

} /* namespace grammar */

