/*
 * LabelFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelFromStringLexer.h"
#include <string>

namespace label {

LabelFromStringLexer::Token LabelFromStringLexer::next(std::istream& in) const {
	LabelFromStringLexer::Token token;
	token.type = TokenType::ERROR;
	token.value = "";
	token.raw = "";
	char character;

L0:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == ' ' || character == '\n' || character == '\t') {
		token.raw += character;
		goto L0;
	} else if(character == '[') {
		token.type = TokenType::SET_BEGIN;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == ']') {
		token.type = TokenType::SET_END;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '<') {
		token.type = TokenType::PAIR_BEGIN;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '>') {
		token.type = TokenType::PAIR_END;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == ',') {
		token.type = TokenType::COMMA;
		token.value += character;
		token.raw += character;
		return token;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
}

void LabelFromStringLexer::putback(std::istream& in, LabelFromStringLexer::Token token) const {
	while(!token.raw.empty()) {
		in.putback(token.raw.back());
		token.raw.pop_back();
	}
	in.clear();
}

} /* namespace label */
