/*
 * LabelToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelToXMLComposer.h"
#include "../std/itos.h"

#include "../XmlApi.hpp"

namespace label {

void LabelToXMLComposer::compose(std::list<sax::Token>& out, const Label& label) const {
	label.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void LabelToXMLComposer::compose(std::list<sax::Token>& out, const LabelBase& label) const {
	label.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void LabelToXMLComposer::compose(std::list<sax::Token>& out, const PrimitiveLabel& label) const {
	out.push_back(sax::Token(alib::Names::LABEL_PRIMITIVE_LABEL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<primitive::Primitive>::compose(out, label.getData());
	out.push_back(sax::Token(alib::Names::LABEL_PRIMITIVE_LABEL, sax::Token::TokenType::END_ELEMENT));
}

void LabelToXMLComposer::compose(std::list<sax::Token>& out, const HexavigesimalLabel& label) const {
	out.push_back(sax::Token(alib::Names::LABEL_HEXAVIGESIMAL_LABEL, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::itos(label.getData()), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::LABEL_HEXAVIGESIMAL_LABEL, sax::Token::TokenType::END_ELEMENT));
}

void LabelToXMLComposer::compose(std::list<sax::Token>& out, const ObjectLabel& label) const {
	out.push_back(sax::Token(alib::Names::LABEL_OBJECT_LABEL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<alib::Object>::compose(out, label.getData());
	out.push_back(sax::Token(alib::Names::LABEL_OBJECT_LABEL, sax::Token::TokenType::END_ELEMENT));
}

void LabelToXMLComposer::compose(std::list<sax::Token>& out, const LabelSetLabel& label) const {
	out.push_back(sax::Token(alib::Names::LABEL_LABEL_SET_LABEL, sax::Token::TokenType::START_ELEMENT));
	for(const Label& innerLabel : label.getData()) {
		alib::xmlApi<label::Label>::compose(out, innerLabel);
	}
	out.push_back(sax::Token(alib::Names::LABEL_LABEL_SET_LABEL, sax::Token::TokenType::END_ELEMENT));
}

void LabelToXMLComposer::compose(std::list<sax::Token>& out, const LabelPairLabel& label) const {
	out.push_back(sax::Token(alib::Names::LABEL_LABEL_PAIR_LABEL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<label::Label>::compose(out, label.getData().first);
	alib::xmlApi<label::Label>::compose(out, label.getData().second);
	out.push_back(sax::Token(alib::Names::LABEL_LABEL_PAIR_LABEL, sax::Token::TokenType::END_ELEMENT));
}

void LabelToXMLComposer::compose(std::list<sax::Token>& out, const UniqueLabel& label) const {
	out.push_back(sax::Token(alib::Names::LABEL_UNIQUE_LABEL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<label::Label>::compose(out, label.getLabel());
	alib::xmlApi<primitive::Integer>::compose(out, label.getId());
	out.push_back(sax::Token(alib::Names::LABEL_UNIQUE_LABEL, sax::Token::TokenType::END_ELEMENT));
}

} /* namespace label */
