/*
 * PrimitiveLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_LABEL_H_
#define PRIMITIVE_LABEL_H_

#include "../primitive/Primitive.h"
#include <ostream>

#include "LabelBase.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class PrimitiveLabel : public std::acceptor<PrimitiveLabel, VisitableLabelBase, std::acceptor<PrimitiveLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	primitive::Primitive primitive;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit PrimitiveLabel(const primitive::Primitive& primitive);

	explicit PrimitiveLabel(primitive::Primitive&& primitive);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const primitive::Primitive& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const PrimitiveLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<PrimitiveLabel>();
	}
};

} /* namespace label */

namespace std {

template<>
struct compare<label::PrimitiveLabel> {
	int operator()(const label::PrimitiveLabel& first, const label::PrimitiveLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* PRIMITIVE_LABEL_H_ */

