/*
 * UniqueLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_ID_LABEL_H_
#define LABEL_ID_LABEL_H_

#include <string>
#include <ostream>

#include "Label.h"
#include "LabelBase.h"
#include <set>
#include "../primitive/Integer.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class UniqueLabel : public std::acceptor<UniqueLabel, VisitableLabelBase, std::acceptor<UniqueLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	Label label;
	primitive::Integer id;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UniqueLabel(const Label& label, primitive::Integer& id);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const Label& getLabel() const;

	const primitive::Integer& getId() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const UniqueLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<UniqueLabel>();
	}
};

} /* namespace label */

namespace std {

template<>
struct compare<label::UniqueLabel> {
	int operator()(const label::UniqueLabel& first, const label::UniqueLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* LABEL_ID_LABEL_H_ */

