/*
 * ObjectBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef OBJECT_BASE_H_
#define OBJECT_BASE_H_

#include "../std/visitor.hpp"
#include "../std/compare.hpp"
#include "../common/base.hpp"

namespace alib {

class Void;

}

namespace exception {

class AlibException;

}

namespace automaton {

class DFA;
class NFA;
class MultiInitialStateNFA;
class EpsilonNFA;
class CompactNFA;
class ExtendedNFA;
class DPDA;
class SinglePopDPDA;
class NPDA;
class SinglePopNPDA;
class InputDrivenNPDA;
class VisiblyPushdownDPDA;
class VisiblyPushdownNPDA;
class RealTimeHeightDeterministicDPDA;
class RealTimeHeightDeterministicNPDA;
class OneTapeDTM;

}

namespace grammar {

class LeftLG;
class LeftRG;
class RightLG;
class RightRG;
class LG;
class CFG;
class EpsilonFreeCFG;
class CNF;
class GNF;
class CSG;
class NonContractingGrammar;
class ContextPreservingUnrestrictedGrammar;
class UnrestrictedGrammar;

}

namespace label {

class PrimitiveLabel;
class ObjectLabel;
class HexavigesimalLabel;
class LabelSetLabel;
class LabelPairLabel;
class UniqueLabel;

}

namespace regexp {

class UnboundedRegExp;
class FormalRegExp;

}

namespace string {

class LinearString;
class CyclicString;
class Epsilon;

}

namespace alphabet {

class StartSymbol;
class LabeledSymbol;
class BlankSymbol;
class BottomOfTheStackSymbol;
class RankedSymbol;
class BarSymbol;
class RankedBarSymbol;
class SubtreeWildcardSymbol;
class SymbolSetSymbol;
class SymbolPairSymbol;
class UniqueSymbol;
class EndSymbol;

}

namespace container {

class ObjectsSet;
class ObjectsVector;
class ObjectsPair;
class ObjectsMap;

}

namespace primitive {

class String;
class Integer;
class Character;
class Unsigned;
class Bool;

}

namespace alib {

class ObjectBase;

typedef std::acceptor_base<ObjectBase,
			Void,
			exception::AlibException,
			automaton::DFA, automaton::NFA, automaton::MultiInitialStateNFA, automaton::EpsilonNFA, automaton::CompactNFA, automaton::ExtendedNFA, automaton::DPDA, automaton::SinglePopDPDA, automaton::InputDrivenNPDA, automaton::VisiblyPushdownDPDA, automaton::VisiblyPushdownNPDA, automaton::RealTimeHeightDeterministicDPDA, automaton::RealTimeHeightDeterministicNPDA, automaton::NPDA, automaton::SinglePopNPDA, automaton::OneTapeDTM,
			grammar::LeftLG, grammar::LeftRG, grammar::RightLG, grammar::RightRG, grammar::LG, grammar::CFG, grammar::EpsilonFreeCFG, grammar::CNF, grammar::GNF, grammar::CSG, grammar::NonContractingGrammar, grammar::ContextPreservingUnrestrictedGrammar, grammar::UnrestrictedGrammar,
			label::PrimitiveLabel, label::HexavigesimalLabel, label::ObjectLabel, label::LabelSetLabel, label::LabelPairLabel, label::UniqueLabel,
			regexp::UnboundedRegExp, regexp::FormalRegExp,
			string::Epsilon, string::LinearString, string::CyclicString,
			alphabet::EndSymbol, alphabet::LabeledSymbol, alphabet::BlankSymbol, alphabet::BottomOfTheStackSymbol, alphabet::RankedSymbol, alphabet::BarSymbol, alphabet::RankedBarSymbol, alphabet::SubtreeWildcardSymbol, alphabet::SymbolPairSymbol, alphabet::SymbolSetSymbol, alphabet::UniqueSymbol, alphabet::StartSymbol,
			container::ObjectsSet, container::ObjectsVector, container::ObjectsPair, container::ObjectsMap,
			primitive::String, primitive::Integer, primitive::Character, primitive::Unsigned, primitive::Bool
	> VisitableObjectBase;

class ObjectBase :
	public alib::base<
			ObjectBase,
			Void,
			exception::AlibException,
			automaton::DFA, automaton::NFA, automaton::MultiInitialStateNFA, automaton::EpsilonNFA, automaton::CompactNFA, automaton::ExtendedNFA, automaton::DPDA, automaton::SinglePopDPDA, automaton::InputDrivenNPDA, automaton::VisiblyPushdownDPDA, automaton::VisiblyPushdownNPDA, automaton::RealTimeHeightDeterministicDPDA, automaton::RealTimeHeightDeterministicNPDA, automaton::NPDA, automaton::SinglePopNPDA, automaton::OneTapeDTM,
			grammar::LeftLG, grammar::LeftRG, grammar::RightLG, grammar::RightRG, grammar::LG, grammar::CFG, grammar::EpsilonFreeCFG, grammar::CNF, grammar::GNF, grammar::CSG, grammar::NonContractingGrammar, grammar::ContextPreservingUnrestrictedGrammar, grammar::UnrestrictedGrammar,
			label::PrimitiveLabel, label::HexavigesimalLabel, label::ObjectLabel, label::LabelSetLabel, label::LabelPairLabel, label::UniqueLabel,
			regexp::UnboundedRegExp, regexp::FormalRegExp,
			string::Epsilon, string::LinearString, string::CyclicString,
			alphabet::EndSymbol, alphabet::LabeledSymbol, alphabet::BlankSymbol, alphabet::BottomOfTheStackSymbol, alphabet::RankedSymbol, alphabet::BarSymbol, alphabet::RankedBarSymbol, alphabet::SubtreeWildcardSymbol, alphabet::SymbolPairSymbol, alphabet::SymbolSetSymbol, alphabet::UniqueSymbol, alphabet::StartSymbol,
			container::ObjectsSet, container::ObjectsVector, container::ObjectsPair, container::ObjectsMap,
			primitive::String, primitive::Integer, primitive::Character, primitive::Unsigned, primitive::Bool
	>, public VisitableObjectBase {
};

} /* namespace alib */

namespace std {

template<>
struct compare<alib::ObjectBase> {
	int operator()(const alib::ObjectBase& first, const alib::ObjectBase& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* OBJECT_BASE_H_ */
