/*
 * Integer.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_INTEGER_H_
#define PRIMITIVE_INTEGER_H_

#include <ostream>

#include "PrimitiveBase.h"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Integer : public std::acceptor<Integer, VisitablePrimitiveBase, std::acceptor<Integer, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	int data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Integer(int data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	int getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const Integer& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<Integer>();
	}
};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::Integer> {
	int operator()(const primitive::Integer& first, const primitive::Integer& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* PRIMITIVE_INTEGER_H_ */
