/*
 * RegExpToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include <algorithm>
#include "RegExpToStringComposer.h"
#include "../StringApi.hpp"

namespace regexp {

void RegExpToStringComposer::Visit(void* userData, const UnboundedRegExp& regexp) const {
	regexp.getRegExp().Accept(userData, *this);
}

void RegExpToStringComposer::Visit(void* userData, const FormalRegExp& regexp) const {
	regexp.getRegExp().Accept(userData, *this);
}

void RegExpToStringComposer::Visit(void* userData, const UnboundedRegExpAlternation& alternation) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);

	if(alternation.getElements().size() == 0) {
		std::get<1>(out) << "#0";
	} else if(alternation.getElements().size() == 1) {
		alternation.getElements()[0]->Accept(userData, *this);
	} else {
		Priority outerPriorityMinimum = std::get<0>(out);
		if(outerPriorityMinimum == Priority::CONCATENATION || outerPriorityMinimum == Priority::FACTOR) std::get<1>(out) << '(';
		bool first = true;
		for (const auto& element : alternation.getElements()) {
			if(first) {
				first = false;
			} else {
				std::get<1>(out) << '+';
			}
			std::get<0>(out) = Priority::ALTERNATION;
			element->Accept(userData, *this);
		}
		if(outerPriorityMinimum == Priority::CONCATENATION || outerPriorityMinimum == Priority::FACTOR) std::get<1>(out) << ')';
	}
}

void RegExpToStringComposer::Visit(void* userData, const UnboundedRegExpConcatenation& concatenation) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);

	Priority outerPriorityMinimum = std::get<0>(out);
	if(concatenation.getElements().size() == 0) {
		std::get<1>(out) << "#E";
	} else if(concatenation.getElements().size() == 1) {
		concatenation.getElements()[0]->Accept(userData, *this);
	} else {
		if(outerPriorityMinimum == Priority::FACTOR) std::get<1>(out) << '(';
		bool first = true;
		for (auto element : concatenation.getElements()) {
			if(first) {
				first = false;
			} else {
				std::get<1>(out) << ' ';
			}
			std::get<0>(out) = Priority::CONCATENATION;
			element->Accept(userData, *this);
		}
		if(outerPriorityMinimum == Priority::FACTOR) std::get<1>(out) << ')';
	}
}

void RegExpToStringComposer::Visit(void* userData, const UnboundedRegExpIteration& iteration) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);

	std::get<0>(out) = Priority::FACTOR;
	iteration.getElement().Accept(userData, *this);
	std::get<1>(out) << "*";
}

void RegExpToStringComposer::Visit(void* userData, const UnboundedRegExpSymbol& symbol) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);

	alib::stringApi<alphabet::Symbol>::compose(std::get<1>(out), symbol.getSymbol());
}

void RegExpToStringComposer::Visit(void* userData, const UnboundedRegExpEpsilon&) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);
	std::get<1>(out) << "#E";
}

void RegExpToStringComposer::Visit(void* userData, const UnboundedRegExpEmpty&) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);
	std::get<1>(out) << "#0";
}

void RegExpToStringComposer::Visit(void* userData, const FormalRegExpAlternation& alternation) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);

	Priority outerPriorityMinimum = std::get<0>(out);
	if(outerPriorityMinimum == Priority::CONCATENATION || outerPriorityMinimum == Priority::FACTOR) std::get<1>(out) << '(';

	std::get<0>(out) = Priority::ALTERNATION;
	alternation.getLeftElement().Accept(userData, *this);
	std::get<1>(out) << '+';
	alternation.getRightElement().Accept(userData, *this);

	if(outerPriorityMinimum == Priority::CONCATENATION || outerPriorityMinimum == Priority::FACTOR) std::get<1>(out) << ')';
}

void RegExpToStringComposer::Visit(void* userData, const FormalRegExpConcatenation& concatenation) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);

	Priority outerPriorityMinimum = std::get<0>(out);
	if(outerPriorityMinimum == Priority::FACTOR) std::get<1>(out) << '(';

	std::get<0>(out) = Priority::CONCATENATION;
	concatenation.getLeftElement().Accept(userData, *this);
	std::get<1>(out) << ' ';
	concatenation.getRightElement().Accept(userData, *this);

	if(outerPriorityMinimum == Priority::FACTOR) std::get<1>(out) << ')';
}

void RegExpToStringComposer::Visit(void* userData, const FormalRegExpIteration& iteration) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);

	std::get<0>(out) = Priority::FACTOR;
	iteration.getElement().Accept(userData, *this);
	std::get<1>(out) << "*";
}

void RegExpToStringComposer::Visit(void* userData, const FormalRegExpSymbol& symbol) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);

	alib::stringApi<alphabet::Symbol>::compose(std::get<1>(out), symbol.getSymbol());
}

void RegExpToStringComposer::Visit(void* userData, const FormalRegExpEpsilon&) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);
	std::get<1>(out) << "#E";
}

void RegExpToStringComposer::Visit(void* userData, const FormalRegExpEmpty&) const {
	std::tuple<Priority&, std::ostream&> &out = *((std::tuple<Priority&, std::ostream&>*) userData);
	std::get<1>(out) << "#0";
}

void RegExpToStringComposer::compose(std::ostream& output, const RegExp& regexp) const {
	Priority tmp = Priority::ALTERNATION;
	std::tuple<Priority&, std::ostream&> out = std::tie(tmp, output);
	regexp.getData().Accept((void*) &out, *this);
}

} /* namespace regexp */
