/*
 * RegExpToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExpToXMLComposer.h"

#include "../XmlApi.hpp"

namespace regexp {

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpAlternation& alternation) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("alternation", sax::Token::TokenType::START_ELEMENT));
	for (const auto& element : alternation.getElements()) {
		element->Accept(userData, *this);
	}
	out.push_back(sax::Token("alternation", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpConcatenation& concatenation) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("concatenation", sax::Token::TokenType::START_ELEMENT));
	for (auto element : concatenation.getElements()) {
		element->Accept(userData, *this);
	}
	out.push_back(sax::Token("concatenation", sax::Token::TokenType::END_ELEMENT));

}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpIteration& iteration) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("iteration", sax::Token::TokenType::START_ELEMENT));
	iteration.getElement().Accept(userData, *this);
	out.push_back(sax::Token("iteration", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpSymbol& symbol) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	alib::xmlApi<alphabet::Symbol>::compose(out, symbol.getSymbol());
}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpEpsilon&) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpEmpty&) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("empty", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("empty", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpAlternation& alternation) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("alternation", sax::Token::TokenType::START_ELEMENT));
	alternation.getLeftElement().Accept(userData, *this);
	alternation.getRightElement().Accept(userData, *this);
	out.push_back(sax::Token("alternation", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpConcatenation& concatenation) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("concatenation", sax::Token::TokenType::START_ELEMENT));
	concatenation.getLeftElement().Accept(userData, *this);
	concatenation.getRightElement().Accept(userData, *this);
	out.push_back(sax::Token("concatenation", sax::Token::TokenType::END_ELEMENT));

}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpIteration& iteration) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("iteration", sax::Token::TokenType::START_ELEMENT));
	iteration.getElement().Accept(userData, *this);
	out.push_back(sax::Token("iteration", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpSymbol& symbol) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	alib::xmlApi<alphabet::Symbol>::compose(out, symbol.getSymbol());
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpEpsilon&) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpEmpty&) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("empty", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("empty", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::compose(std::list<sax::Token>& out, const RegExp& regexp) const {
	regexp.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void RegExpToXMLComposer::compose(std::list<sax::Token>& out, const RegExpBase& regexp) const {
	regexp.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void RegExpToXMLComposer::compose(std::list<sax::Token>& out, const UnboundedRegExp& regexp) const {
	out.push_back(sax::Token(alib::Names::REGEXP_UNBOUNDED_REGEXP, sax::Token::TokenType::START_ELEMENT));
	{
		out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
		for (const auto& symbol: regexp.getAlphabet()) {
			alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
		}
		out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));
	}
	regexp.getRegExp().Accept((void*) &out, *this);
	out.push_back(sax::Token(alib::Names::REGEXP_UNBOUNDED_REGEXP, sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::compose(std::list<sax::Token>& out, const FormalRegExp& regexp) const {
	out.push_back(sax::Token(alib::Names::REGEXP_FORMAL_REGEXP, sax::Token::TokenType::START_ELEMENT));
	{
		out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
		for (const auto& symbol: regexp.getAlphabet()) {
			alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
		}
		out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));
	}
	regexp.getRegExp().Accept((void*) &out, *this);
	out.push_back(sax::Token(alib::Names::REGEXP_FORMAL_REGEXP, sax::Token::TokenType::END_ELEMENT));
}

} /* namespace regexp */
