/*
 * UnboundedRegExpEpsilon.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#include "UnboundedRegExpEpsilon.h"
#include "../formal/FormalRegExpEpsilon.h"
#include <sstream>

namespace regexp {

UnboundedRegExpEpsilon::UnboundedRegExpEpsilon() {
  // so that default constructor is available
}

UnboundedRegExpEpsilon::UnboundedRegExpEpsilon(const UnboundedRegExpEpsilon&) {
  // so that copy constructor is available
}

UnboundedRegExpEpsilon::UnboundedRegExpEpsilon(UnboundedRegExpEpsilon&&) noexcept {
	this->attachRegExp(NULL);
}

UnboundedRegExpEpsilon& UnboundedRegExpEpsilon::operator =(const UnboundedRegExpEpsilon&) {
  //this is actually different than default implementation
  return *this;
}

UnboundedRegExpEpsilon& UnboundedRegExpEpsilon::operator =(UnboundedRegExpEpsilon&&) noexcept {
  //this is actually different than default implementation
  return *this;
}

UnboundedRegExpElement* UnboundedRegExpEpsilon::clone() const {
	return new UnboundedRegExpEpsilon(*this);
}

UnboundedRegExpElement* UnboundedRegExpEpsilon::plunder() && {
	return new UnboundedRegExpEpsilon(std::move(*this));
}

FormalRegExpElement* UnboundedRegExpEpsilon::cloneAsFormal() const {
	return new FormalRegExpEpsilon();
}

int UnboundedRegExpEpsilon::compare(const UnboundedRegExpEpsilon&) const {
	  return 0;
}

void UnboundedRegExpEpsilon::operator>>(std::ostream& out) const {
	out << "(UnboundedRegExpEpsilon)";
}

bool UnboundedRegExpEpsilon::testSymbol( const alphabet::Symbol & ) const {
	return false;
}

bool UnboundedRegExpEpsilon::attachRegExp(const UnboundedRegExp * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	return true;
}

void UnboundedRegExpEpsilon::computeMinimalAlphabet( std::set<alphabet::Symbol>&) const {

}

UnboundedRegExpEpsilon::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace regexp */
