/*
 * UnboundedRegExpEpsilon.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef UNBOUNDED_REG_EXP_EPSILON_H_
#define UNBOUNDED_REG_EXP_EPSILON_H_

#include "UnboundedRegExpElement.h"

namespace regexp {

/**
 * Represents epsilon in the regular expression.
 */
class UnboundedRegExpEpsilon: public std::acceptor<UnboundedRegExpEpsilon, UnboundedRegExpElement, UnboundedRegExpElement> {
protected:
	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual FormalRegExpElement* cloneAsFormal() const;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc UnboundedRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const UnboundedRegExp * regexp );
	
	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:

	explicit UnboundedRegExpEpsilon();
	UnboundedRegExpEpsilon(const UnboundedRegExpEpsilon& other);
	UnboundedRegExpEpsilon(UnboundedRegExpEpsilon&& other) noexcept;
	UnboundedRegExpEpsilon& operator=(const UnboundedRegExpEpsilon& other);
	UnboundedRegExpEpsilon& operator=(UnboundedRegExpEpsilon&& other) noexcept;

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* clone() const;
	
	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement* plunder() &&;

	virtual int compare(const UnboundedRegExpElement& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const UnboundedRegExpEpsilon&) const;
	
	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<UnboundedRegExpEpsilon>();
	}
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_EPSILON_H_ */
