/*
 * LinearString.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef LINEAR_STRING_H_
#define LINEAR_STRING_H_

#include <iostream>
#include <set>
#include <vector>

#include "../std/visitor.hpp"
#include "../alphabet/Symbol.h"
#include "StringBase.h"
#include "common/StringAlphabet.h"
#include "Epsilon.h"

namespace string {

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
class LinearString : public std::acceptor<LinearString, VisitableStringBase, std::acceptor<LinearString, alib::VisitableObjectBase, StringBase> >, public StringAlphabet {
	std::vector<alphabet::Symbol> m_Data;

public:
	explicit LinearString();
	explicit LinearString(const std::set<alphabet::Symbol>& alphabet, const std::vector<alphabet::Symbol>& data);
	explicit LinearString(std::set<alphabet::Symbol>&& alphabet, std::vector<alphabet::Symbol>&& data);
	explicit LinearString(const std::vector<alphabet::Symbol>& data);
	explicit LinearString(std::vector<alphabet::Symbol>&& data);
	explicit LinearString(const std::string& str);
	explicit LinearString(const Epsilon& epsilon);

	virtual StringBase* clone() const;
	virtual StringBase* plunder() &&;

	virtual bool removeSymbolFromAlphabet(const alphabet::Symbol & symbol);

	/**
	 * @param element to append
	 */
	void appendSymbol(const alphabet::Symbol& symbol);
	
	void appendSymbol(alphabet::Symbol&& symbol);

	/**
	 * @return List of symbols forming string (const version).
	 */
	const std::vector<alphabet::Symbol>& getContent() const;

	void setContent(std::vector<alphabet::Symbol> data);

	/**
	 * @return true if string is an empty word (vector length is 0)
	 */
	bool isEmpty() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const LinearString& other) const;
	virtual int compare(const CyclicString& other) const;
	virtual int compare(const Epsilon& other) const;

	virtual void operator >>(std::ostream& out) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<LinearString>();
	}
};

} /* namespace string */

namespace std {

template<>
struct compare<::string::LinearString> {
	int operator()(const ::string::LinearString& first, const ::string::LinearString& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* LINEAR_STRING_H_ */

