#include <list>
#include "SymbolTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "alphabet/Symbol.h"
#include "alphabet/LabeledSymbol.h"
#include "alphabet/BlankSymbol.h"

#include "factory/XmlDataFactory.hpp"
#include "factory/StringDataFactory.hpp"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))
#define CPPUNIT_EXCLUSIVE_OR(x, y) CPPUNIT_ASSERT((!(x) && (y)) || ((x) && !(y)))

CPPUNIT_TEST_SUITE_REGISTRATION( SymbolTest );

void SymbolTest::setUp() {
}

void SymbolTest::tearDown() {
}

void SymbolTest::testCopyConstruct() {
	alphabet::Symbol symbol = alphabet::symbolFrom("1");

	alphabet::Symbol symbol2(symbol);

	CPPUNIT_ASSERT( symbol == symbol2 );

	alphabet::Symbol symbol3(std::move(symbol));

	CPPUNIT_ASSERT( symbol2 == symbol3 );
}

void SymbolTest::testEqual() {
	{
		std::string input = "a";
		alphabet::Symbol symbol = alib::StringDataFactory::fromString<alphabet::Symbol>(input);

		std::string output = alib::StringDataFactory::toString(symbol);

		CPPUNIT_ASSERT( input == output );

		alphabet::Symbol symbol2 = alib::StringDataFactory::fromString<alphabet::Symbol>(output);

		CPPUNIT_ASSERT( symbol == symbol2 );
	}
	{
		std::string input = "'aaaa'";
		alphabet::Symbol symbol = alib::StringDataFactory::fromString<alphabet::Symbol>(input);

		std::string output = alib::StringDataFactory::toString(symbol);

		CPPUNIT_ASSERT( input == output );

		alphabet::Symbol symbol2 = alib::StringDataFactory::fromString<alphabet::Symbol>(output);

		CPPUNIT_ASSERT( symbol == symbol2 );
	}
	{
		std::string input = "#B";
		alphabet::Symbol symbol = alib::StringDataFactory::fromString<alphabet::Symbol>(input);

		std::string output = alib::StringDataFactory::toString(symbol);

		CPPUNIT_ASSERT( input == output );

		alphabet::Symbol symbol2 = alib::StringDataFactory::fromString<alphabet::Symbol>(output);

		CPPUNIT_ASSERT( symbol == symbol2 );
	}
}

void SymbolTest::testXMLParser() {

	alphabet::Symbol symbol = alphabet::symbolFrom("1");

	{
		std::list<sax::Token> tokens = alib::XmlDataFactory::toTokens(symbol);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		std::list<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		alphabet::Symbol symbol2 = alib::XmlDataFactory::fromTokens<alphabet::Symbol>(tokens2);

		CPPUNIT_ASSERT( symbol == symbol2 );
	}
	{
		std::string tmp = alib::XmlDataFactory::toString(symbol);
		alphabet::Symbol symbol2 = alib::XmlDataFactory::fromString<alphabet::Symbol>(tmp);

		CPPUNIT_ASSERT( symbol == symbol2 );
	}
}

void SymbolTest::testOrder() {
	alphabet::LabeledSymbol ls("a");
	alphabet::BlankSymbol bs;
	alphabet::StartSymbol ss;
	alphabet::EndSymbol es;

	CPPUNIT_EXCLUSIVE_OR( bs < ls, ls < bs);

	CPPUNIT_ASSERT(es < ls); //End symbol typically denoted by '$' is lexicographically smaller than any other symbol
	CPPUNIT_ASSERT(ls < ss); //Start symbol typically denoted by '^' is lexicographically bigger than any other symbol
;
}

